Linux cgroups development
 help / color / mirror / Atom feed
* [PATCH v2] cpuset: relax the overlap check for cgroup-v2
@ 2025-11-13 13:14 Sun Shaojie
  2025-11-13 14:57 ` Waiman Long
                   ` (2 more replies)
  0 siblings, 3 replies; 25+ messages in thread
From: Sun Shaojie @ 2025-11-13 13:14 UTC (permalink / raw)
  To: llong
  Cc: cgroups, chenridong, hannes, linux-kernel, linux-kselftest,
	mkoutny, shuah, sunshaojie, tj

In cgroup v2, a mutual overlap check is required when at least one of two
cpusets is exclusive. However, this check should be relaxed and limited to
cases where both cpusets are exclusive.

The table 1 shows the partition states of A1 and B1 after each step before
applying this patch.

Table 1: Before applying the patch
 Step                                       | A1's prstate | B1's prstate |
 #1> mkdir -p A1                            | member       |              |
 #2> echo "0-1" > A1/cpuset.cpus            | member       |              |
 #3> echo "root" > A1/cpuset.cpus.partition | root         |              |
 #4> mkdir -p B1                            | root         | member       |
 #5> echo "0-3" > B1/cpuset.cpus            | root invalid | member       |
 #6> echo "root" > B1/cpuset.cpus.partition | root invalid | root invalid |

After step #5, A1 changes from "root" to "root invalid" because its CPUs
(0-1) overlap with those requested by B1 (0-3). However, B1 can actually
use CPUs 2-3, so it would be more reasonable for A1 to remain as "root."

This patch relaxes the exclusive cpuset check for cgroup v2 while
preserving the current cgroup v1 behavior.

Signed-off-by: Sun Shaojie <sunshaojie@kylinos.cn>

---
v1 -> v2:
  - Keeps the current cgroup v1 behavior unchanged
  - Link: https://lore.kernel.org/cgroups/c8e234f4-2c27-4753-8f39-8ae83197efd3@redhat.com
---
 kernel/cgroup/cpuset.c                            |  9 +++++++--
 tools/testing/selftests/cgroup/test_cpuset_prs.sh | 10 +++++-----
 2 files changed, 12 insertions(+), 7 deletions(-)

diff --git a/tools/testing/selftests/cgroup/test_cpuset_prs.sh b/tools/testing/selftests/cgroup/test_cpuset_prs.sh
index a17256d9f88a..903dddfe88d7 100755
--- a/tools/testing/selftests/cgroup/test_cpuset_prs.sh
+++ b/tools/testing/selftests/cgroup/test_cpuset_prs.sh
@@ -269,7 +269,7 @@ TEST_MATRIX=(
 	" C0-3:S+ C1-3:S+ C2-3     .    X2-3   X3:P2    .      .     0 A1:0-2|A2:3|A3:3 A1:P0|A2:P2 3"
 	" C0-3:S+ C1-3:S+ C2-3     .    X2-3   X2-3  X2-3:P2   .     0 A1:0-1|A2:1|A3:2-3 A1:P0|A3:P2 2-3"
 	" C0-3:S+ C1-3:S+ C2-3     .    X2-3   X2-3 X2-3:P2:C3 .     0 A1:0-1|A2:1|A3:2-3 A1:P0|A3:P2 2-3"
-	" C0-3:S+ C1-3:S+ C2-3   C2-3     .      .      .      P2    0 A1:0-3|A2:1-3|A3:2-3|B1:2-3 A1:P0|A3:P0|B1:P-2"
+	" C0-3:S+ C1-3:S+ C2-3   C2-3     .      .      .      P2    0 A1:0-1|A2:1|A3:1|B1:2-3 A1:P0|A3:P0|B1:P2 2-3"
 	" C0-3:S+ C1-3:S+ C2-3   C4-5     .      .      .      P2    0 B1:4-5 B1:P2 4-5"
 	" C0-3:S+ C1-3:S+ C2-3    C4    X2-3   X2-3  X2-3:P2   P2    0 A3:2-3|B1:4 A3:P2|B1:P2 2-4"
 	" C0-3:S+ C1-3:S+ C2-3    C4    X2-3   X2-3 X2-3:P2:C1-3 P2  0 A3:2-3|B1:4 A3:P2|B1:P2 2-4"
@@ -318,7 +318,7 @@ TEST_MATRIX=(
 	# Invalid to valid local partition direct transition tests
 	" C1-3:S+:P2 X4:P2  .      .      .      .      .      .     0 A1:1-3|XA1:1-3|A2:1-3:XA2: A1:P2|A2:P-2 1-3"
 	" C1-3:S+:P2 X4:P2  .      .      .    X3:P2    .      .     0 A1:1-2|XA1:1-3|A2:3:XA2:3 A1:P2|A2:P2 1-3"
-	"  C0-3:P2   .      .    C4-6   C0-4     .      .      .     0 A1:0-4|B1:4-6 A1:P-2|B1:P0"
+	"  C0-3:P2   .      .    C4-6   C0-4     .      .      .     0 A1:0-4|B1:5-6 A1:P2|B1:P0 0-4"
 	"  C0-3:P2   .      .    C4-6 C0-4:C0-3  .      .      .     0 A1:0-3|B1:4-6 A1:P2|B1:P0 0-3"
 
 	# Local partition invalidation tests
@@ -388,10 +388,10 @@ TEST_MATRIX=(
 	"  C0-1:S+  C1      .    C2-3     .      P2     .      .     0 A1:0-1|A2:1 A1:P0|A2:P-2"
 	"  C0-1:S+ C1:P2    .    C2-3     P1     .      .      .     0 A1:0|A2:1 A1:P1|A2:P2 0-1|1"
 
-	# A non-exclusive cpuset.cpus change will invalidate partition and its siblings
-	"  C0-1:P1   .      .    C2-3   C0-2     .      .      .     0 A1:0-2|B1:2-3 A1:P-1|B1:P0"
+	# A non-exclusive cpuset.cpus change will not invalidate partition and its siblings
+	"  C0-1:P1   .      .    C2-3   C0-2     .      .      .     0 A1:0-2|B1:3 A1:P1|B1:P0"
 	"  C0-1:P1   .      .  P1:C2-3  C0-2     .      .      .     0 A1:0-2|B1:2-3 A1:P-1|B1:P-1"
-	"   C0-1     .      .  P1:C2-3  C0-2     .      .      .     0 A1:0-2|B1:2-3 A1:P0|B1:P-1"
+	"   C0-1     .      .  P1:C2-3  C0-2     .      .      .     0 A1:0-1|B1:2-3 A1:P0|B1:P1"
 
 	# cpuset.cpus can overlap with sibling cpuset.cpus.exclusive but not subsumed by it
 	"   C0-3     .      .    C4-5     X5     .      .      .     0 A1:0-3|B1:4-5"
diff --git a/kernel/cgroup/cpuset.c b/kernel/cgroup/cpuset.c
index 52468d2c178a..3240b3ab5998 100644
--- a/kernel/cgroup/cpuset.c
+++ b/kernel/cgroup/cpuset.c
@@ -592,8 +592,13 @@ static inline bool cpusets_are_exclusive(struct cpuset *cs1, struct cpuset *cs2)
  */
 static inline bool cpus_excl_conflict(struct cpuset *cs1, struct cpuset *cs2)
 {
-	/* If either cpuset is exclusive, check if they are mutually exclusive */
-	if (is_cpu_exclusive(cs1) || is_cpu_exclusive(cs2))
+	/* If both cpusets are exclusive, check if they are mutually exclusive */
+	if (is_cpu_exclusive(cs1) && is_cpu_exclusive(cs2))
+		return !cpusets_are_exclusive(cs1, cs2);
+
+	/* In cgroup-v1, if either cpuset is exclusive, check if they are mutually exclusive */
+	if (!is_in_v2_mode() &&
+	    (is_cpu_exclusive(cs1) != is_cpu_exclusive(cs2)))
 		return !cpusets_are_exclusive(cs1, cs2);
 
 	/* Exclusive_cpus cannot intersect */
-- 
2.25.1


^ permalink raw reply related	[flat|nested] 25+ messages in thread

end of thread, other threads:[~2025-11-17 18:43 UTC | newest]

Thread overview: 25+ messages (download: mbox.gz follow: Atom feed
-- links below jump to the message on this page --
2025-11-13 13:14 [PATCH v2] cpuset: relax the overlap check for cgroup-v2 Sun Shaojie
2025-11-13 14:57 ` Waiman Long
2025-11-13 17:07 ` Michal Koutný
2025-11-14  1:29   ` Chen Ridong
2025-11-14 16:14     ` Michal Koutný
2025-11-15  0:58       ` Chen Ridong
2025-11-14  0:50 ` Chen Ridong
2025-11-14  5:53   ` Sun Shaojie
2025-11-14  6:24   ` Sun Shaojie
2025-11-14 16:15     ` Michal Koutný
2025-11-15  2:01       ` Chen Ridong
2025-11-15  9:31         ` [PATCH -next] cpuset: treate root invalid trialcs as exclusive Chen Ridong
2025-11-16 14:08           ` [PATCH v2] cpuset: relax the overlap check for cgroup-v2 Sun Shaojie
2025-11-17  4:35           ` [PATCH -next] cpuset: treate root invalid trialcs as exclusive Sun Shaojie
2025-11-17  6:23             ` Chen Ridong
2025-11-17  6:53               ` Sun Shaojie
2025-11-17  7:30                 ` Chen Ridong
2025-11-17 15:56           ` Waiman Long
2025-11-15  9:51         ` [PATCH v2] cpuset: relax the overlap check for cgroup-v2 Chen Ridong
2025-11-15 11:24       ` Sun Shaojie
2025-11-14  6:33   ` Sun Shaojie
2025-11-14  6:59     ` Chen Ridong
2025-11-15  6:02   ` Sun Shaojie
2025-11-15  7:41     ` Chen Ridong
2025-11-17 18:43     ` Waiman Long

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox